# FRACTAL MUSIC
#
# Author:  Alexander M. Henderson
# Created: 21 August 2014
#
# This script computes the orbit of a complex number and converts that
# orbit into a sequence of notes that can be loosely interpreted as
# "music".
#
# This script requires the music21 library:  http://web.mit.edu/music21/

from cmath import *
from music21 import *
import Gnuplot

# --- Change These Values --- #
real = 0.3750
imag = 0.2850
N = 64
# --------------------------- #

c = complex(real,imag)
phase(c)

z = [c]
for i in range(0,N):
	z.append(z[i]*z[i]+c)

# The complex plane is divided into 24 wedges, each representing one
# one note in a chromatic scale.  The note assigned to each point in the
# orbit is determined by which of these wedges the point lives in.
notes = []
for i in z:
	arg = phase(i)
	if arg < -11*pi/12:
		notes.append("C")
	elif arg < -10*pi/12:
		notes.append("C#")
	elif arg < -9*pi/12:
		notes.append("D")
	elif arg < -8*pi/12:
		notes.append("D#")
	elif arg < -7*pi/12:
		notes.append("E")
	elif arg < -6*pi/12:
		notes.append("F")
	elif arg < -5*pi/12:
		notes.append("F#")
	elif arg < -4*pi/12:
		notes.append("G")
	elif arg < -3*pi/12:
		notes.append("G#")
	elif arg < -2*pi/12:
		notes.append("A")
	elif arg < -1*pi/12:
		notes.append("A#")
	elif arg < 0:
		notes.append("B")
	elif arg < pi/12:
		notes.append("c")
	elif arg < 2*pi/12:
		notes.append("c#")
	elif arg < 3*pi/12:
		notes.append("d")
	elif arg < 4*pi/12:
		notes.append("d#")
	elif arg < 5*pi/12:
		notes.append("e")
	elif arg < 6*pi/12:
		notes.append("f")
	elif arg < 7*pi/12:
		notes.append("f#")
	elif arg < 8*pi/12:
		notes.append("g")
	elif arg < 9*pi/12:
		notes.append("g#")
	elif arg < 10*pi/12:
		notes.append("a")
	elif arg < 11*pi/12:
		notes.append("a#")
	else:
		notes.append("b")

# The duration of each note is determined by the modulus of the 
# corresponding point in the orbit.  Points closer to the origin have
# a longer duration.
durations = []
for i in z:
	mod = abs(i)
	if mod < 0.5:
		durations.append("2")
	elif mod < 1.0:
		durations.append("4")
	elif mod < 1.5:
		durations.append("8")
	else:
		durations.append("16")

music_str = "tinyNotation: 4/4 "
for i in range(0,N):
	music_str = music_str + notes[i] + durations[i] + " "

music = converter.parse(music_str)
music.show('midi')

points = []
for i in z:
	points.append([i.real,i.imag])
print points

# Just for giggles, we plot the orbit.
g = Gnuplot.Gnuplot(persist=1)
#g("set terminal x11")
g("set style line 1 linecolor rgb 'blue' lw 2")
g("set size square")
g("set yrange [-2:2]")
g("set xrange [-2:2]")
g("set grid")
g.title("Orbit of ({0}) + ({1})i ({2} iterations)".format(str(real),str(imag),str(N)))
d = Gnuplot.Data(points,with_="lines ls 1")
g.plot(d)
